# -*- coding: UTF-8 -*-
import logging


import django.test as unittest

from log_storage.models import Log


class TestLogRecordingToFile(unittest.TestCase):
    def setUp(self):
        self.log = Log(save_file=True)
        self.logger = logging.getLogger("test." + __name__)
        self.logger.setLevel(logging.DEBUG)

    def test_logging(self):
        with self.log:
            self.logger.info("info")
            self.logger.debug("debug")
        self.assertIn(u"info", self.log.log_data)
        self.assertIn(u"debug", self.log.log_data)
        self.assertIn(u"test.log_storage.tests", self.log.log_data)

    def test_persistence(self):
        with self.log:
            self.logger.info("info")
            self.logger.debug("debug")
        self.log = Log.objects.get(pk=self.log.pk)
        self.assertIn(u"info", self.log.log_data)
        self.assertIn(u"debug", self.log.log_data)
        self.assertIn(u"test.log_storage.tests", self.log.log_data)

    def test_persistence_backward_compatibility(self):
        with self.log:
            self.logger.info("info")
            self.logger.debug("debug")
        self.log = Log.objects.get(pk=self.log.pk)
        self.assertTrue(self.log.filename.startswith("logs/"))
        self.log.filename = self.log.filename[5:]
        self.assertIn(u"info", self.log.log_data)
        self.assertIn(u"debug", self.log.log_data)
        self.assertIn(u"test.log_storage.tests", self.log.log_data)

    def test_log_data(self):
        """Test that calling log.log_data before logging anything won't throw error."""
        self.assertEqual("", self.log.log_data)

    def test_logging_unicode(self):
        with self.log:
            self.logger.info(u"инфо")
            self.logger.debug(u"дебуг")
        self.log = Log.objects.get(pk=self.log.pk)
        self.assertIn(u"инфо", self.log.log_data)
        self.assertIn(u"дебуг", self.log.log_data)


class TestLogRecordingToDb(unittest.TestCase):
    def setUp(self):
        self.log = Log(save_file=False)
        self.logger = logging.getLogger("test." + __name__)
        self.logger.setLevel(logging.DEBUG)

    def test_logging(self):
        with self.log:
            self.logger.info("info")
            self.logger.debug("debug")
        self.assertIn(u"info", self.log.log_data)
        self.assertIn(u"debug", self.log.log_data)
        self.assertIn(u"test.log_storage.tests", self.log.log_data)

    def test_persistence(self):
        with self.log:
            self.logger.info("info")
            self.logger.debug("debug")
        self.log = Log.objects.get(pk=self.log.pk)
        self.assertIn(u"info", self.log.log_data)
        self.assertIn(u"debug", self.log.log_data)
        self.assertIn(u"test.log_storage.tests", self.log.log_data)

    def test_log_data(self):
        """Test that calling log.log_data before logging anything won't throw error."""
        self.assertEqual("", self.log.log_data)

    def test_logging_unicode(self):
        with self.log:
            self.logger.info(u"инфо")
            self.logger.debug(u"дебуг")
        self.log = Log.objects.get(pk=self.log.pk)
        self.assertIn(u"инфо", self.log.log_data)
        self.assertIn(u"дебуг", self.log.log_data)



#!/usr/bin/env python

import os
from setuptools import setup

long_description = "Please see our GitHub README"
if os.path.exists("README.md"):
    long_description = open("README.md").read()

setup(
    name="log_storage",
    version="0.10.0",
    license="MIT",
    description="Log handler to store messages with a database record and optional file storage.",
    keywords=["Django", "Log", "App"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Virtualstock",
    author_email="development.team@virtualstock.com",
    url="https://github.com/virtualstock/log_storage/",
    packages=["log_storage", "log_storage.migrations"],
    include_package_data=True,
    install_requires=["django", "django-private-storage"],
)



import os, time

from private_storage.storage import private_storage

from django.conf import settings
from django.core.files.base import ContentFile
from django.db import models
from django.utils.encoding import smart_str


class BaseLog(models.Model):
    save_file = models.BooleanField(default=True)
    db_log_data = models.TextField()
    filename = models.CharField(max_length=100)
    created = models.DateTimeField(auto_now_add=True)
    prefix = ""
    suffix = ".log"

    class Meta:
        abstract = True

    def _get_desired_filename(self):
        path = getattr(settings, "LOG_DATA_DIR", "logs")
        filename = "{}_{}{}".format(
            self.prefix, time.strftime("%Y%m%d_%H%M%S_%f"), self.suffix
        )
        return os.path.join(path, filename)

    @property
    def log_data(self):
        if self.save_file:
            if self.filename:
                if private_storage.exists(self.filename):
                    file_path = self.filename
                else:
                    # Backwards compatability with 0.7.0
                    path = getattr(settings, "LOG_DATA_DIR", "logs")
                    file_path = os.path.join(path, self.filename)
                with private_storage.open(file_path, "rb") as f:
                    return f.read().decode("utf-8")
            else:
                return u""
        else:
            return self.db_log_data or u""

    _handler = None

    def make_file_handler(self):
        from logging import StreamHandler

        class FileStream(object):
            def __init__(stream):
                stream.contents = ContentFile(b"")
                stream.contents.open("wb")

            def write(stream, data):
                stream.contents.write(smart_str(data).encode(("utf-8")))

        class FileStreamHandler(StreamHandler):
            def close(handler):
                handler.stream.contents.seek(0)
                self.filename = private_storage.save(
                    self._get_desired_filename(), handler.stream.contents
                )
                super(FileStreamHandler, handler).close()

        return FileStreamHandler(FileStream())

    def make_db_handler(self):
        from logging import StreamHandler

        self.db_log_data = self.db_log_data or u""

        class DbStream(object):
            def write(stream, data):
                self.db_log_data += data

            def flush(stream):
                self.save()

            close = flush

        return StreamHandler(DbStream())

    @property
    def formatter(self):
        from logging import Formatter

        return Formatter("%(asctime)s   %(levelname)s   %(name)s   %(message)s")

    def make_handler(self):
        if self.save_file:
            handler = self.make_file_handler()
        else:
            handler = self.make_db_handler()
        handler.formatter = self.formatter
        return handler

    def __enter__(self):
        import logging

        self._handler = self.make_handler()
        logging.root.addHandler(self._handler)

    def __exit__(self, *tpl):
        import logging

        logging.root.removeHandler(self._handler)
        self._handler.close()
        self._handler = None
        self.save()


class Log(BaseLog):
    pass



from django.contrib import admin
from .models import Log


class LogAdmin(admin.ModelAdmin):
    readonly_fields = ("log_data", "db_log_data", "filename", "save_file")
    list_display = ("filename", "created", "save_file")
    list_filter = ("created", "save_file")
    search_fields = ("filename",)


admin.site.register(Log, LogAdmin)
